{% extends "base.html" %} {% block content %}

<div class="page1024">
  <h1>Dashboard</h1>
  <ul class="dashboard-list">
    {% for name, pretty_name in pages %}
    <li>
      <a href="/{{ name }}">
        <img src="../static/img/icon-button-admin.svg" /> {{ pretty_name }}</a
      >
    </li>
    {% endfor %}
  </ul>
</div>

{% endblock %}
