{% extends "base.html" %}

{% block content %}    

    <div class="page1024">
        <h1>Page Not Found</h1>
        <p>Sorry, but the page you were trying to view does not exist.</p>
    </div>

{% endblock %}