{% extends "base.html" %} {% block extra_libraries %}
<script
  src="https://code.jquery.com/jquery-3.3.1.min.js"
  integrity="sha256-FgpCb/KJQlLNfOu91ta32o/NMZxltwRo8QtmkMRdAu8="
  crossorigin="anonymous"
></script>
<script>
  window.jQuery ||
    document.write(
      '<script src="/static/js/vendor/jquery-3.3.1.min.js"><\/script>'
    );
</script>
<script src="https://cdn.jsdelivr.net/npm/jquery-validation@1.19.0/dist/jquery.validate.js"></script>
{% endblock %} {% block content %}
<div id="loginFormPanel" class="login-form-panel">
  <h1>Reset Password</h1>
  <div class="back-button">
    <a href="/"> << Return to Dashboard</a>
  </div>
  <form
    id="resetPasswordForm"
    class="reset-password-form"
    action=""
    method="post"
  >
    {{ form.hidden_tag() }}
    <div class="inputBlock">
      {{ form.password.label }} {{ form.password(size=32) }}
    </div>
    <div class="inputBlock">
      {{ form.password2.label }} {{ form.password2(size=32) }}
    </div>
    {{ form.submit() }}
  </form>
</div>

<script>
  $("#resetPasswordForm").validate({
    rules: {
      password: {
        required: true,
      },
      password2: {
        required: true,
        equalTo: "#password",
      },
    },
  });
</script>

{% endblock %}
