@media screen {
  @import url("https://fonts.googleapis.com/css2?family=Lato:ital,wght@0,300;0,400;0,700;0,900;1,300;1,700;1,900&display=swap");

  html,
  body {
    font-family: "Lato", sans-serif;
    width: 100%;
    height: 100%;
    font-size: 16px;
  }
  :root {
    --white: #ffffff;
    --black: #000000;
    --green: #699f56;
    --blue: #34566f;
    --light-blue: #00bbff;
    --grey: #959595;
    --grey2: #d8d8d8;
    --red: #aa4926;
    --orange: #cc7832;
  }

  /* Main Panel - To Show Full Background*/
  /* ============================================================= */
  .main {
    height: 100%;
    width: 100%;
  }

  .main:after {
    content: "";
    display: block;
    position: fixed;
    top: 0;
    left: 0;
    background-image: url("/static/img/background.jpg");
    background-size: 100% 100%;
    width: 100%;
    height: 100%;
    opacity: 0.3;
    z-index: -1;
    background-repeat: no-repeat;
    background-position: center;
    background-attachment: fixed;
  }

  /* Header containing logo and buttons*/
  /* ============================================================= */
  .header-container {
    height: 3.75em;
    background-color: var(--black);
    padding-top: 0.625em;
  }

  .header-container .content-wrapper {
    flex-direction: row;
  }

  header .logo-image {
    width: 40px;
    height: 40px;
  }

  .content-wrapper {
    margin: 0 auto;
    padding: 0 0.5em;
    width: 60em;
    overflow-y: hidden;
  }

  .row {
    display: flex;
  }

  .header-left {
    display: flex;
  }
  .header-right {
    margin-left: auto;
    text-align: center;
  }

  h1.logo-text {
    font-size: 1.4em;
    font-weight: 400;
    text-align: left;
    margin: 0;
    padding: 0;
    border: 0px none;
    position: relative;
    left: 0.75em;
    top: 0.55em;
  }

  h1.logo-text a {
    text-decoration: none;
    color: var(--white);
  }

  .header-right a {
    text-decoration: none;
    color: var(--white);
  }

  .header-right img {
    width: 20px;
    height: 20px;
  }

  span.logout-link {
    font-size: 0.7em;
    text-transform: uppercase;
    display: block;
    position: relative;
    bottom: 0px;
  }

  header .header-buttons {
    display: flex;
  }

  .buttonBlock {
    display: flex;
    flex-direction: column;
    align-items: center;
    margin-top: 0.375em;
  }

  .buttonBlock:last-child {
    margin-left: 1.25em;
  }

  /* Area to flash messages to user*/
  /* ============================================================= */
  .flashes {
    border-top: 1px solid silver;
    padding: 10px;
    box-shadow: 0px 2px 5px -2px rgba(0, 0, 0, 0.6);
    background: white;
    display: flex;
    flex-direction: row;
    justify-content: space-between;
    align-items: center;
    z-index: 1;
    color: green;
  }

  /* Full Page With Panel with 20px margin to show pretty background
    /* ============================================================= */
  .fullPage {
    flex-grow: 1;
    flex-shrink: 1;
    margin: 20px;

    border: 1px solid silver;
    padding: 20px;
    box-shadow: 0px 0px 5px rgba(0, 0, 0, 0.3);
    background: white;
    display: flex;
    flex-direction: column;
    align-items: stretch;
  }

  /* Full Page With Panel with 20px margin to show pretty background
    /* ============================================================= */
  .page1024 {
    margin: 0 auto;
    margin-top: 20px;
    border: 1px solid silver;
    padding: 20px;
    box-shadow: 0px 0px 5px rgba(0, 0, 0, 0.3);
    background: var(--white);
    width: 100%;
    overflow-y: hidden;
  }

  /* Global styles*/
  /* ============================================================= */
  h1 {
    font-weight: 700;
    font-size: 28px;
    margin-bottom: 20px;
    text-align: center;
    border-bottom: 1px solid silver;
    padding-bottom: 10px;
  }

  /* Login Form */
  /* ============================================================= */

  .login-form-panel {
    width: 350px;
    margin: 0 auto;
    margin-top: 50px;

    border: 1px solid silver;
    padding: 30px;
    padding-top: 25px;
    padding-bottom: 10px;
    box-shadow: 0px 0px 5px rgba(0, 0, 0, 0.3);
    background-color: var(--white);
  }

  .login-form-panel h1 {
    font-weight: 700;
    font-size: 28px;
    margin-bottom: 20px;
    text-align: center;
    border-bottom: 1px solid silver;
    padding-bottom: 10px;
  }

  .login-form-link {
    color: var(--blue);
    margin: 1em 0em;
    display: block;
  }

  .login-form-panel label {
    display: inline-block;
    margin-bottom: 5px;
    font-weight: 600;
  }

  .login-form-panel .inputBlock {
    margin-bottom: 15px;
  }

  .login-form-panel .labelLine {
    display: flex;
    flex-direction: row;
    justify-content: space-between;
  }

  .login-form-panel input[type="text"],
  input[type="password"] {
    width: 100%;
    font-size: 16px;
    padding: 5px;
    font-weight: 400;
  }

  .login-form-panel input[type="submit"] {
    width: 100%;
    padding: 10px 0px;
    margin: 0px;
    margin-left: -1px;
    margin-bottom: 10px;
    border: 0px none;
    border-radius: 10px;
    background-color: var(--blue);
    font-size: 1em;
    cursor: pointer;
    color: var(--white);
  }

  .login-form-panel input[type="submit"]:hover {
    filter: brightness(85%);
  }

  .login-form-panel input[type="checkbox"] {
    padding: 0px;
    margin: 0px;
  }

  .remember-me {
    display: flex;
    flex-direction: row;
    align-items: center;
    margin-bottom: 15px;
  }

  .login-form label[for="remember_me"] {
    font-size: 14px;
    margin: 0px;
    margin-left: 5px;
    font-weight: 400;
  }

  .login-form-panel a {
    display: block;
    margin-bottom: 10px;
    font-size: 14px;
    margin-top: 12px;
  }

  form .error {
    color: red;
    font-size: 14px;
  }

  /* Dashboard page */
  /* ============================================================= */

  .dashboard-list {
    width: 21em;
    margin: 0 auto;
  }

  .dashboard-list li {
    margin-bottom: 1.25em;
  }

  .dashboard-list li a {
    padding: 0.4em 0.875em 0.8em 1.625em;
    background-color: var(--blue);
    color: var(--white);
    display: block;
    text-decoration: none;
    border-radius: 0.6em;
    box-sizing: border-box;
  }

  .dashboard-list li a > img {
    position: relative;
    top: 0.25em;
    right: 0.8em;
  }

  .dashboard-list li a:hover {
    filter: brightness(85%);
  }

  .back-button {
    margin-bottom: 1em;
  }

  .form-section .form-group {
    margin-bottom: 1.3em;
  }

  .form-section .form-group input {
    display: block;
    margin: 0.7em 0;
    border: 1px solid var(--grey);
    padding: 0.5em 1em;
  }

  .form-section .checkbox {
    margin-bottom: 1.3em;
  }

  .form-section .btn.btn-success {
    padding: 0.75em 1.375em;
    background-color: var(--blue);
    color: var(--white);
    display: block;
    text-decoration: none;
    border-radius: 0.6em;
    box-shadow: none;
    border: 0px none;
    cursor: pointer;
    font-size: 1em;
  }

  .form-body-content {
    overflow-x: scroll;
    overflow-y: hidden;
  }

  .form-section .btn.btn-success:hover {
    filter: brightness(85%);
  }

  .back-button {
    margin-bottom: 1em;
  }

  .back-button a {
    padding: 8px 0px;
    color: var(--blue);
    text-decoration: none;
    font-size: 1em;
    font-style: italic;
  }

  .back-button a:hover {
    opacity: 0.8;
  }

  /* Responsive styles */
  /* ============================================================= */

  /* Tablet */
  @media (max-width: 960px) {
    .content-wrapper {
      max-width: 40em;
      flex-direction: row;
      width: 100%;
    }
    .dashboard-list {
      width: 100%;
    }
  }

  /* Desktop-small */
  @media (min-width: 1260px) {
    .content-wrapper {
      max-width: 1260px;
      width: 100%;
    }
    .dashboard-list {
      width: 25em;
    }
  }

  /* Desktop-large  */
  @media (min-width: 1600px) {
    .content-wrapper {
      max-width: 1600px;
      width: 100%;
    }
    .dashboard-list {
      width: 28em;
    }
  }
}
