"""Add user attributes

Revision ID: 1b1532ba9506
Revises: 1582bc5883ac
Create Date: 2021-04-22 18:42:10.038438

"""
from alembic import op
import sqlalchemy as sa


# revision identifiers, used by Alembic.
revision = '1b1532ba9506'
down_revision = '1582bc5883ac'
branch_labels = None
depends_on = None


def upgrade():
    # ### commands auto generated by Alembic - please adjust! ###
    op.add_column('user', sa.Column('attributes', sa.String(length=512), nullable=False, server_default="{}"))
    # ### end Alembic commands ###


def downgrade():
    # ### commands auto generated by Alembic - please adjust! ###
    op.drop_column('user', 'attributes')
    # ### end Alembic commands ###
