{% extends "base.html" %} {% block extra_libraries %}
<script
  src="https://code.jquery.com/jquery-3.6.0.min.js"
  integrity="sha256-/xUj+3OJU5yExlq6GSYGSHk7tPXikynS7ogEvDej/m4="
  crossorigin="anonymous"
></script>
{% endblock %} {% block content %}

<div class="page1024">
  <h1>{{title}}</h1>
  <div class="back-button">
    <a href="/"> << Return to Dashboard</a>
  </div>
  <p>{{ body|safe }}</p>
</div>

{% endblock %}
